"""
Copyright (C) 2019-2020 Cumulus Networks,  Inc. all rights reserved

This software is subject to the Cumulus Networks End User License Agreement available
at the following locations:

Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
Cumulus Linux systems: /usr/share/cumulus/EULA.txt
"""

import logging
import re
from pprint import pformat

"""
parser for ethtool swpXX
"""

this_log = logging.getLogger(__name__)
debug    = this_log.debug
info     = this_log.info
warn     = this_log.warning
error    = this_log.error

name = "ethtool"


def __separate_keys(dct, name, i=2, prefix='other_'):
    other_name = prefix + name
    if other_name in dct and dct[other_name]:
        for s in dct[other_name].split():
            dct[name + str(i)] = s
            i += 1
        del dct[other_name]

def parse(command, output):
    """
    Return a dictionary or list containing the values parsed from the output
    If parse failed, return None
    """

    # Use regex named groups. Always use \s instead of ' ' for spaces in output
    pattern = re.compile(r'''
                        \s*Settings\s+for\s+(?P<kernel_port>\w+):
                        \s+Supported\sports:\s+\[(?P<supp_ports>.*)\]
                        \s+Supported\slink\smodes:\s+(?P<supp_modes1>Not\s[Rr]eported|[0-9]+base[248a-zA-Z/_]+)
                        (?P<other_supp_modes>(?:\s*[0-9]+base[248a-zA-Z/_]+){,31}?)
                        \s+Supported\spause\sframe\suse:\s+(?P<supp_pause>[a-zA-Z\- ]+)  #Symmetric Receive-only
                        \s+Supports\sauto-negotiation:\s+(?P<supp_autoneg>\w+)
                        (\s+Supported \sFEC\smodes:
                        (?P<other_supp_fec_modes>(?:\s+[0-9a-zA-Z /]+){,3}?))?
                        \s+Advertised\slink\smodes:\s+(?P<adv_modes>Not\s[Rr]eported|[0-9]+base[248a-zA-Z/_]+) #Need 1000Mb/s and Not Reported
                        (?P<other_adv_modes>(?:\s*[0-9]+base[248a-zA-Z/_]+){,14}?)
                        \s+Advertised\spause\sframe\suse:\s+(?P<adv_pause>[a-zA-Z\- ]+)
                        \s+Advertised\sauto-negotiation:\s+(?P<adv_autoneg>\w+)
                        (\s+Advertised\sFEC\smodes:\s+(?P<adv_fec_modes>[0-9a-zA-Z /]+))?
                        (\s+Link\spartner\sadvertised\slink\smodes:\s+(?P<lp_adv_modes>Not\s[Rr]eported|[0-9]+base[248a-zA-Z/_]+))?
                        (?P<other_lp_adv_modes>(?:\s*[0-9]+base[248a-zA-Z/_]+){,31}?)
                        (\s+Link\spartner\sadvertised\spause\sframe\suse:\s+(?P<lp_adv_pause>[a-zA-Z\- ]+))?
                        (\s+Link\spartner\sadvertised\sauto-negotiation:\s+(?P<lp_adv_autoneg>\w+))?
                        (\s+Link\spartner\sadvertised\sFEC\smodes:\s+(?P<lp_adv_fec_modes>[0-9a-zA-Z /]+))?
                        \s+Speed:\s+(?P<speed>[\w!\/]+)  # Need to pick up Unknown!
                        \s+Duplex:\s(?P<duplex>[\w!\/ \(\)]+)  # Need to pick up Unknown!
                        \s+Port:\s+(?P<port_type>[a-zA-Z ]+)
                        \s+PHYAD:\s(?P<phyad>\w+)
                        \s+Transceiver:\s(?P<transceiver>\w+)
                        \s+Auto-negotiation:\s(?P<autoneg>\w+)
                        (\s+MDI-X:\s(?P<mdix>[onfautUnkw \(\)]+))?  #Field not always here. on/off (auto), Unknown
                        (\s+Current\smessage\slevel:\s+(?P<curr_msg_lvl>[a-zA-Z0-9\(\)\-_\s]+))? #not always here in CL3
                        \s+Link\sdetected:\s+(?P<linkstate>\w+)\s*
                        ''', re.X)

    debug('Ethtool_out:\n' + output)
    match = pattern.match(output)

    if match:
        ethtool_dct = match.groupdict()

        # Separates groups
        __separate_keys(ethtool_dct, 'supp_modes')
        __separate_keys(ethtool_dct, 'supp_fec_modes', i=1)
        __separate_keys(ethtool_dct, 'adv_modes')
        __separate_keys(ethtool_dct, 'lp_adv_modes')

        debug('ethtool output in list format:' + pformat(match.group(0)))

    else:
        error('No ethtool match: {}'.format(output))
        return None

    return ethtool_dct
